@@ -80,7 +80,6 @@ import org.apache.qpid.jms.test.testpeer.matchers.types.EncodedAmqpValueMatcher;
 import org.apache.qpid.proton.amqp.Binary;
 import org.apache.qpid.proton.amqp.Symbol;
 import org.apache.qpid.proton.amqp.UnsignedInteger;
-import org.junit.Ignore;
 import org.junit.Test;
 
 public class SessionIntegrationTest extends QpidJmsTestCase {
@@ -456,7 +455,6 @@ public class SessionIntegrationTest extends QpidJmsTestCase {
         }
     }
 
-    @Ignore // TODO: The *test* needs Proton 0.9, enable when upgrading.
     @Test(timeout = 5000)
     public void testCreateAnonymousProducerTargetContainsNoTypeCapabilityWhenAnonymousRelayNodeIsSupported() throws Exception {
         try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
@@ -601,7 +599,7 @@ public class SessionIntegrationTest extends QpidJmsTestCase {
     public void testCreateDurableTopicSubscriberFailsIfConnectionDoesntHaveExplicitClientID() throws Exception {
         try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
             // Create a connection without an explicit clientId
-            Connection connection = testFixture.establishConnecton(testPeer, null, null, null, false);
+            Connection connection = testFixture.establishConnecton(testPeer, false, null, null, null, false);
             connection.start();
 
             testPeer.expectBegin(true);
@@ -648,7 +646,6 @@ public class SessionIntegrationTest extends QpidJmsTestCase {
         }
     }
 
-    @Ignore // TODO: The *test* needs Proton 0.9, enable when upgrading.
     @Test(timeout = 5000)
     public void testCreateAnonymousProducerWhenAnonymousRelayNodeIsSupported() throws Exception {
         try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
@@ -697,13 +694,11 @@ public class SessionIntegrationTest extends QpidJmsTestCase {
         }
     }
 
-    @Ignore // TODO: The *test* needs Proton 0.9, enable when upgrading.
     @Test(timeout = 5000)
     public void testCreateAnonymousProducerFailsWhenAnonymousRelayNodeIsSupportedButLinkRefusedAndAttachResponseWriteIsNotDeferred() throws Exception {
         doCreateAnonymousProducerFailsWhenAnonymousRelayNodeIsSupportedButLinkRefusedTestImpl(false);
     }
 
-    @Ignore // TODO: The *test* needs Proton 0.9, enable when upgrading.
     @Test(timeout = 5000)
     public void testCreateAnonymousProducerFailsWhenAnonymousRelayNodeIsSupportedButLinkRefusedAndAttachResponseWriteIsDeferred() throws Exception {
         doCreateAnonymousProducerFailsWhenAnonymousRelayNodeIsSupportedButLinkRefusedTestImpl(true);
@@ -1233,6 +1228,8 @@ public class SessionIntegrationTest extends QpidJmsTestCase {
 
     @Test(timeout = 5000)
     public void testRemotelyEndSessionWithProducer() throws Exception {
+        final String BREAD_CRUMB = "ErrorMessage";
+
         try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
             Connection connection = testFixture.establishConnecton(testPeer);
 
@@ -1241,7 +1238,7 @@ public class SessionIntegrationTest extends QpidJmsTestCase {
 
             // Create a producer, then remotely end the session afterwards.
             testPeer.expectSenderAttach();
-            testPeer.remotelyEndLastOpenedSession(true);
+            testPeer.remotelyEndLastOpenedSession(true, 0, AmqpError.RESOURCE_DELETED, BREAD_CRUMB);
 
             Queue queue = session.createQueue("myQueue");
             final MessageProducer producer = session.createProducer(queue);
@@ -1256,12 +1253,28 @@ public class SessionIntegrationTest extends QpidJmsTestCase {
                     try {
                         producer.getDestination();
                     } catch (IllegalStateException jmsise) {
-                        return true;
+                        if (jmsise.getCause() != null) {
+                            String message = jmsise.getCause().getMessage();
+                            return message.contains(AmqpError.RESOURCE_DELETED.toString()) &&
+                                   message.contains(BREAD_CRUMB);
+                        } else {
+                            return false;
+                        }
                     }
                     return false;
                 }
             }, 2000, 10));
 
+            // Verify the session is now marked closed
+            try {
+                session.getAcknowledgeMode();
+                fail("Expected ISE to be thrown due to being closed");
+            } catch (IllegalStateException jmsise) {
+                String message = jmsise.getCause().getMessage();
+                assertTrue(message.contains(AmqpError.RESOURCE_DELETED.toString()));
+                assertTrue(message.contains(BREAD_CRUMB));
+            }
+
             // Try closing it explicitly, should effectively no-op in client.
             // The test peer will throw during close if it sends anything.
             producer.close();
@@ -1270,6 +1283,8 @@ public class SessionIntegrationTest extends QpidJmsTestCase {
 
     @Test(timeout = 5000)
     public void testRemotelyEndSessionWithConsumer() throws Exception {
+        final String BREAD_CRUMB = "ErrorMessage";
+
         try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
             Connection connection = testFixture.establishConnecton(testPeer);
 
@@ -1279,7 +1294,7 @@ public class SessionIntegrationTest extends QpidJmsTestCase {
             // Create a consumer, then remotely end the session afterwards.
             testPeer.expectReceiverAttach();
             testPeer.expectLinkFlow();
-            testPeer.remotelyEndLastOpenedSession(true);
+            testPeer.remotelyEndLastOpenedSession(true, 0, AmqpError.RESOURCE_DELETED, BREAD_CRUMB);
 
             Queue queue = session.createQueue("myQueue");
             final MessageConsumer consumer = session.createConsumer(queue);
@@ -1292,12 +1307,28 @@ public class SessionIntegrationTest extends QpidJmsTestCase {
                     try {
                         consumer.getMessageListener();
                     } catch (IllegalStateException jmsise) {
-                        return true;
+                        if (jmsise.getCause() != null) {
+                            String message = jmsise.getCause().getMessage();
+                            return message.contains(AmqpError.RESOURCE_DELETED.toString()) &&
+                                   message.contains(BREAD_CRUMB);
+                        } else {
+                            return false;
+                        }
                     }
                     return false;
                 }
             }, 2000, 10));
 
+            // Verify the session is now marked closed
+            try {
+                session.getAcknowledgeMode();
+                fail("Expected ISE to be thrown due to being closed");
+            } catch (IllegalStateException jmsise) {
+                String message = jmsise.getCause().getMessage();
+                assertTrue(message.contains(AmqpError.RESOURCE_DELETED.toString()));
+                assertTrue(message.contains(BREAD_CRUMB));
+            }
+
             // Try closing it explicitly, should effectively no-op in client.
             // The test peer will throw during close if it sends anything.
             consumer.close();
