@@ -27,12 +27,16 @@ import static org.hamcrest.Matchers.notNullValue;
 import static org.hamcrest.Matchers.nullValue;
 
 import java.io.IOException;
+import java.net.Socket;
 import java.util.ArrayList;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.TimeUnit;
 
+import javax.net.ssl.SSLContext;
+
 import org.apache.qpid.jms.provider.amqp.AmqpConnection;
 import org.apache.qpid.jms.provider.amqp.AmqpTemporaryDestination;
 import org.apache.qpid.jms.provider.amqp.message.AmqpDestinationHelper;
@@ -112,10 +116,17 @@ public class TestAmqpPeer implements AutoCloseable
     private byte[] _deferredBytes;
     private int _lastInitiatedChannel = -1;
     private UnsignedInteger _lastInitiatedLinkHandle = null;
+    private int advertisedIdleTimeout = 0;
+    private int _emptyFrameCount = 0;
 
     public TestAmqpPeer() throws IOException
     {
-        _driverRunnable = new TestAmqpPeerRunner(this);
+        this(null, false);
+    }
+
+    public TestAmqpPeer(SSLContext context, boolean needClientCert) throws IOException
+    {
+        _driverRunnable = new TestAmqpPeerRunner(this, context, needClientCert);
         _driverThread = new Thread(_driverRunnable, "MockAmqpPeerThread");
         _driverThread.start();
     }
@@ -161,11 +172,35 @@ public class TestAmqpPeer implements AutoCloseable
         return _driverRunnable.getException();
     }
 
+    public void setSuppressReadExceptionOnClose(boolean suppress)
+    {
+        _driverRunnable.setSuppressReadExceptionOnClose(suppress);
+    }
+
     public int getServerPort()
     {
         return _driverRunnable.getServerPort();
     }
 
+    public Socket getClientSocket()
+    {
+        return _driverRunnable.getClientSocket();
+    }
+
+    public int getAdvertisedIdleTimeout()
+    {
+        return advertisedIdleTimeout;
+    }
+
+    public void setAdvertisedIdleTimeout(int advertisedIdleTimeout)
+    {
+        this.advertisedIdleTimeout = advertisedIdleTimeout;
+    }
+
+    public int getEmptyFrameCount() {
+        return _emptyFrameCount;
+    }
+
     public void receiveHeader(byte[] header)
     {
         Handler handler = getFirstHandler();
@@ -200,6 +235,12 @@ public class TestAmqpPeer implements AutoCloseable
         }
     }
 
+    public void receiveEmptyFrame(int type, int channel)
+    {
+        _emptyFrameCount ++;
+        LOGGER.debug("Received empty frame");
+    }
+
     private void removeFirstHandler()
     {
         synchronized(_handlersLock)
@@ -265,6 +306,11 @@ public class TestAmqpPeer implements AutoCloseable
         _driverRunnable.sendBytes(header);
     }
 
+    public void sendEmptyFrame(boolean deferWrite)
+    {
+        sendFrame(FrameType.AMQP, 0, null, null, deferWrite);
+    }
+
     public void sendFrame(FrameType type, int channel, DescribedType frameDescribedType, Binary framePayload, boolean deferWrite)
     {
         if(channel < 0)
@@ -306,7 +352,24 @@ public class TestAmqpPeer implements AutoCloseable
         }
     }
 
+    private OpenFrame createOpenFrame()
+    {
+        OpenFrame openFrame = new OpenFrame();
+        openFrame.setContainerId("test-amqp-peer-container-id");
+        if(advertisedIdleTimeout != 0)
+        {
+            openFrame.setIdleTimeOut(UnsignedInteger.valueOf(advertisedIdleTimeout));
+        }
+
+        return openFrame;
+    }
+
     public void expectAnonymousConnect(boolean authorize)
+    {
+        expectAnonymousConnect(authorize, null);
+    }
+
+    public void expectAnonymousConnect(boolean authorize, Matcher<?> idleTimeoutMatcher)
     {
         SaslMechanismsFrame saslMechanismsFrame = new SaslMechanismsFrame().setSaslServerMechanisms(Symbol.valueOf("ANONYMOUS"));
         addHandler(new HeaderHandlerImpl(AmqpHeader.SASL_HEADER, AmqpHeader.SASL_HEADER,
@@ -316,6 +379,7 @@ public class TestAmqpPeer implements AutoCloseable
 
         addHandler(new SaslInitMatcher()
             .withMechanism(equalTo(Symbol.valueOf("ANONYMOUS")))
+            .withInitialResponse(nullValue())
             .onSuccess(new AmqpPeerRunnable()
             {
                 @Override
@@ -332,12 +396,21 @@ public class TestAmqpPeer implements AutoCloseable
 
         addHandler(new HeaderHandlerImpl(AmqpHeader.HEADER, AmqpHeader.HEADER));
 
-        addHandler(new OpenMatcher()
+        OpenFrame openFrame = createOpenFrame();
+
+        OpenMatcher openMatcher = new OpenMatcher()
             .withContainerId(notNullValue(String.class))
             .onSuccess(new FrameSender(
                     this, FrameType.AMQP, 0,
-                    new OpenFrame().setContainerId("test-amqp-peer-container-id"),
-                    null)));
+                    openFrame,
+                    null));
+
+        if(idleTimeoutMatcher !=null)
+        {
+            openMatcher.withIdleTimeOut(idleTimeoutMatcher);
+        }
+
+        addHandler(openMatcher);
     }
 
     /**
@@ -353,6 +426,7 @@ public class TestAmqpPeer implements AutoCloseable
 
         addHandler(new SaslInitMatcher()
             .withMechanism(equalTo(Symbol.valueOf("EXTERNAL")))
+            .withInitialResponse(equalTo(new Binary(new byte[0])))
             .onSuccess(new AmqpPeerRunnable()
             {
                 @Override
@@ -369,11 +443,13 @@ public class TestAmqpPeer implements AutoCloseable
 
         addHandler(new HeaderHandlerImpl(AmqpHeader.HEADER, AmqpHeader.HEADER));
 
+        OpenFrame openFrame = createOpenFrame();
+
         addHandler(new OpenMatcher()
             .withContainerId(notNullValue(String.class))
             .onSuccess(new FrameSender(
                     this, FrameType.AMQP, 0,
-                    new OpenFrame().setContainerId("test-amqp-peer-container-id"),
+                    openFrame,
                     null)));
     }
 
@@ -415,8 +491,7 @@ public class TestAmqpPeer implements AutoCloseable
 
         addHandler(new HeaderHandlerImpl(AmqpHeader.HEADER, AmqpHeader.HEADER));
 
-        OpenFrame open = new OpenFrame();
-        open.setContainerId("test-amqp-peer-container-id");
+        OpenFrame open = createOpenFrame();
         if(serverCapabilities != null)
         {
             open.setOfferedCapabilities(serverCapabilities);
@@ -443,10 +518,70 @@ public class TestAmqpPeer implements AutoCloseable
         addHandler(openMatcher);
     }
 
+    // TODO - Reject any incoming connection using the supplied information
+    public void rejectConnect(Symbol errorType, String errorMessage, Map<String, Object> errorInfo) {
+
+        SaslMechanismsFrame saslMechanismsFrame = new SaslMechanismsFrame().setSaslServerMechanisms(Symbol.valueOf("ANONYMOUS"));
+        addHandler(new HeaderHandlerImpl(AmqpHeader.SASL_HEADER, AmqpHeader.SASL_HEADER,
+                                            new FrameSender(
+                                                    this, FrameType.SASL, 0,
+                                                    saslMechanismsFrame, null)));
+
+        addHandler(new SaslInitMatcher()
+            .withMechanism(equalTo(Symbol.valueOf("ANONYMOUS")))
+            .withInitialResponse(nullValue())
+            .onSuccess(new AmqpPeerRunnable()
+            {
+                @Override
+                public void run()
+                {
+                    TestAmqpPeer.this.sendFrame(
+                            FrameType.SASL, 0,
+                            new SaslOutcomeFrame().setCode(UnsignedByte.valueOf((byte)0)),
+                            null,
+                            false);
+                    _driverRunnable.expectHeader();
+                }
+            }));
+
+        addHandler(new HeaderHandlerImpl(AmqpHeader.HEADER, AmqpHeader.HEADER));
+
+        Map<Symbol, Object> properties = new HashMap<Symbol, Object>();
+        properties.put(Symbol.valueOf("amqp:connection-establishment-failed"), true);
+
+        OpenFrame open = createOpenFrame();
+        open.setProperties(properties);
+
+        addHandler(new OpenMatcher()
+            .withContainerId(notNullValue(String.class))
+            .onSuccess(new FrameSender(this, FrameType.AMQP, CONNECTION_CHANNEL, open, null)));
+
+        // Now generate the Close with the supplied error
+        final CloseFrame closeFrame = new CloseFrame();
+        if (errorType != null) {
+            org.apache.qpid.jms.test.testpeer.describedtypes.Error detachError = new org.apache.qpid.jms.test.testpeer.describedtypes.Error();
+            detachError.setCondition(errorType);
+            detachError.setDescription(errorMessage);
+            detachError.setInfo(errorInfo);
+            closeFrame.setError(detachError);
+        }
+
+        CompositeAmqpPeerRunnable comp = insertCompsiteActionForLastHandler();
+        final FrameSender closeSender = new FrameSender(this, FrameType.AMQP, CONNECTION_CHANNEL, closeFrame, null);
+        comp.add(closeSender);
+
+        addHandler(new CloseMatcher().withError(Matchers.nullValue()));
+    }
+
     public void expectClose()
+    {
+        expectClose(Matchers.nullValue());
+    }
+
+    public void expectClose(Matcher<?> errorMatcher)
     {
         addHandler(new CloseMatcher()
-            .withError(Matchers.nullValue())
+            .withError(errorMatcher)
             .onSuccess(new FrameSender(this, FrameType.AMQP, 0,
                     new CloseFrame(),
                     null)));
@@ -581,7 +716,6 @@ public class TestAmqpPeer implements AutoCloseable
                 if (!refuseLink) {
                     Target t = (Target) createTargetObjectFromDescribedType(attachMatcher.getReceivedTarget());
                     t.setAddress(dynamicAddress);
-                    trimTargetCapabilities(t);
                     attachResponse.setTarget(t);
                 } else {
                     attachResponse.setTarget(null);
@@ -695,11 +829,11 @@ public class TestAmqpPeer implements AutoCloseable
                 attachResponseSender.setChannel(attachMatcher.getActualChannel());
                 attachResponse.setHandle(receivedHandle);
                 attachResponse.setName(attachMatcher.getReceivedName());
-                attachResponse.setSource(trimSourceOutcomesCapabilities(createSourceObjectFromDescribedType(attachMatcher.getReceivedSource())));
+                attachResponse.setSource(createSourceObjectFromDescribedType(attachMatcher.getReceivedSource()));
                 if(refuseLink) {
                     attachResponse.setTarget(null);
                 } else {
-                    attachResponse.setTarget(trimTargetCapabilities(createTargetObjectFromDescribedType(attachMatcher.getReceivedTarget())));
+                    attachResponse.setTarget(createTargetObjectFromDescribedType(attachMatcher.getReceivedTarget()));
                 }
 
                 _lastInitiatedLinkHandle = (UnsignedInteger) receivedHandle;
@@ -809,7 +943,7 @@ public class TestAmqpPeer implements AutoCloseable
                 if(refuseLink) {
                     attachResponse.setSource(null);
                 } else {
-                    attachResponse.setSource(trimSourceOutcomesCapabilities(createSourceObjectFromDescribedType(attachMatcher.getReceivedSource())));
+                    attachResponse.setSource(createSourceObjectFromDescribedType(attachMatcher.getReceivedSource()));
                 }
 
                 _lastInitiatedLinkHandle = (UnsignedInteger) receivedHandle;
@@ -1209,39 +1343,26 @@ public class TestAmqpPeer implements AutoCloseable
         return new Source(sourceFields.toArray());
     }
 
-    private Source trimSourceOutcomesCapabilities(final Source source) {
-        // TODO: this is a workaround for Proton 0.8 compatibility for the tests.
-        // Remove method and update callers when upgrading to 0.9.
-        source.setOutcomes(null);
-        source.setCapabilities(null);
-        return source;
-    }
-
-    private Object trimTargetCapabilities(final Object target) {
-        // TODO: this is a workaround for Proton 0.8 compatibility for the tests.
-        // Remove method and update callers when upgrading to 0.9.
-        if (target instanceof Target) {
-            ((Target) target).setCapabilities(null);
-        } else if (target instanceof Coordinator) {
-            ((Coordinator) target).setCapabilities(null);
-        } else {
-            throw new IllegalArgumentException("Unexpected object type: " + target.getClass());
-        }
-
-        return target;
-    }
-
     public void remotelyEndLastOpenedSession(boolean expectEndResponse) {
-        remotelyEndLastOpenedSession(expectEndResponse, 0);
+        remotelyEndLastOpenedSession(expectEndResponse, 0, null, null);
     }
 
-    public void remotelyEndLastOpenedSession(boolean expectEndResponse, final long delayBeforeSend) {
+    public void remotelyEndLastOpenedSession(boolean expectEndResponse, long delayBeforeSend) {
+        remotelyEndLastOpenedSession(expectEndResponse, 0, null, null);
+    }
+
+    public void remotelyEndLastOpenedSession(boolean expectEndResponse, final long delayBeforeSend, Symbol errorType, String errorMessage) {
         synchronized (_handlersLock) {
             CompositeAmqpPeerRunnable comp = insertCompsiteActionForLastHandler();
 
             // Now generate the End for the appropriate session
             final EndFrame endFrame = new EndFrame();
-            // TODO: add an optional error msg+condition?
+            if (errorType != null) {
+                org.apache.qpid.jms.test.testpeer.describedtypes.Error detachError = new org.apache.qpid.jms.test.testpeer.describedtypes.Error();
+                detachError.setCondition(errorType);
+                detachError.setDescription(errorMessage);
+                endFrame.setError(detachError);
+            }
 
             int channel = -1;
             final FrameSender frameSender = new FrameSender(this, FrameType.AMQP, channel, endFrame, null);
@@ -1272,13 +1393,27 @@ public class TestAmqpPeer implements AutoCloseable
     }
 
     public void remotelyCloseConnection(boolean expectCloseResponse) {
+        remotelyCloseConnection(expectCloseResponse, null, null, null);
+    }
+
+    public void remotelyCloseConnection(boolean expectCloseResponse, Symbol errorType, String errorMessage) {
+        remotelyCloseConnection(expectCloseResponse, errorType, errorMessage, null);
+    }
+
+    public void remotelyCloseConnection(boolean expectCloseResponse, Symbol errorType, String errorMessage, Map<String, Object> info) {
         synchronized (_handlersLock) {
             // Prepare a composite to insert this action at the end of the handler sequence
             CompositeAmqpPeerRunnable comp = insertCompsiteActionForLastHandler();
 
             // Now generate the Close
             final CloseFrame closeFrame = new CloseFrame();
-            // TODO: add an optional error msg+condition?
+            if (errorType != null) {
+                org.apache.qpid.jms.test.testpeer.describedtypes.Error detachError = new org.apache.qpid.jms.test.testpeer.describedtypes.Error();
+                detachError.setCondition(errorType);
+                detachError.setDescription(errorMessage);
+                detachError.setInfo(info);
+                closeFrame.setError(detachError);
+            }
 
             final FrameSender frameSender = new FrameSender(this, FrameType.AMQP, CONNECTION_CHANNEL, closeFrame, null);
 
@@ -1293,13 +1428,22 @@ public class TestAmqpPeer implements AutoCloseable
     }
 
     public void remotelyDetachLastOpenedLinkOnLastOpenedSession(boolean expectDetachResponse, boolean closed) {
+        remotelyDetachLastOpenedLinkOnLastOpenedSession(expectDetachResponse, closed, null, null);
+    }
+
+    public void remotelyDetachLastOpenedLinkOnLastOpenedSession(boolean expectDetachResponse, boolean closed, Symbol errorType, String errorMessage) {
         synchronized (_handlersLock) {
             CompositeAmqpPeerRunnable comp = insertCompsiteActionForLastHandler();
 
             // Now generate the Detach for the appropriate link on the appropriate session
             final DetachFrame detachFrame = new DetachFrame();
             detachFrame.setClosed(closed);
-            // TODO: add an optional error msg+condition?
+            if (errorType != null) {
+                org.apache.qpid.jms.test.testpeer.describedtypes.Error detachError = new org.apache.qpid.jms.test.testpeer.describedtypes.Error();
+                detachError.setCondition(errorType);
+                detachError.setDescription(errorMessage);
+                detachFrame.setError(detachError);
+            }
 
             // The response frame channel will be dynamically set based on the previous frames. Using the -1 is an illegal placeholder.
             final FrameSender frameSender = new FrameSender(this, FrameType.AMQP, -1, detachFrame, null);
@@ -1371,4 +1515,12 @@ public class TestAmqpPeer implements AutoCloseable
             comp.add(transferSender);
         }
     }
+
+    public void runAfterLastHandler(AmqpPeerRunnable action) {
+        synchronized (_handlersLock) {
+            // Prepare a composite to insert this action at the end of the handler sequence
+            CompositeAmqpPeerRunnable comp = insertCompsiteActionForLastHandler();
+            comp.add(action);
+        }
+    }
 }
