@@ -25,6 +25,10 @@ import java.net.ServerSocket;
 import java.net.Socket;
 import java.nio.ByteBuffer;
 
+import javax.net.ssl.SSLContext;
+import javax.net.ssl.SSLServerSocket;
+import javax.net.ssl.SSLServerSocketFactory;
+
 import org.apache.qpid.proton.amqp.Binary;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -43,18 +47,37 @@ class TestAmqpPeerRunner implements Runnable
 
     private final Object _inputHandlingLock = new Object();
     private final TestFrameParser _testFrameParser;
+    private volatile boolean _suppressReadExceptionOnClose;
 
     private volatile Throwable _throwable;
 
-    public TestAmqpPeerRunner(TestAmqpPeer peer) throws IOException
+    public TestAmqpPeerRunner(TestAmqpPeer peer, SSLContext sslContext, boolean needClientCert) throws IOException
     {
-        _serverSocket = new ServerSocket(useFixedPort ? PORT : 0);
+        int port = useFixedPort ? PORT : 0;
+
+        if (sslContext == null)
+        {
+            _serverSocket = new ServerSocket(port);
+        }
+        else
+        {
+            SSLServerSocketFactory socketFactory = sslContext.getServerSocketFactory();
+            _serverSocket = socketFactory.createServerSocket(port);
+
+            SSLServerSocket sslServerSocket = (SSLServerSocket) _serverSocket;
+            if (needClientCert)
+            {
+                sslServerSocket.setNeedClientAuth(true);
+            }
+        }
+
         _testFrameParser = new TestFrameParser(peer);
     }
 
     @Override
     public void run()
     {
+        boolean attemptingRead = false;
         try
         (
             Socket clientSocket = _serverSocket.accept();
@@ -69,8 +92,10 @@ class TestAmqpPeerRunner implements Runnable
             byte[] networkInputBytes = new byte[1024];
 
             LOGGER.trace("Attempting read");
+            attemptingRead = true;
             while((bytesRead = networkInputStream.read(networkInputBytes)) != -1)
             {
+                attemptingRead = false;
                 //prevent stop() from killing the socket while the frame parser might be using it handling input
                 synchronized(_inputHandlingLock)
                 {
@@ -81,13 +106,18 @@ class TestAmqpPeerRunner implements Runnable
                     _testFrameParser.input(networkInputByteBuffer);
                 }
                 LOGGER.trace("Attempting read");
+                attemptingRead = true;
             }
 
             LOGGER.trace("Exited read loop");
         }
         catch (Throwable t)
         {
-            if(!_serverSocket.isClosed())
+            if (attemptingRead && _suppressReadExceptionOnClose && t instanceof IOException)
+            {
+                LOGGER.debug("Caught exception during read, suppressing as expected: " + t, t);
+            }
+            else if(!_serverSocket.isClosed())
             {
                 LOGGER.error("Problem in peer", t);
                 _throwable = t;
@@ -170,4 +200,14 @@ class TestAmqpPeerRunner implements Runnable
 
         return -1;
     }
+
+    public Socket getClientSocket()
+    {
+        return _clientSocket;
+    }
+
+    public void setSuppressReadExceptionOnClose(boolean suppress)
+    {
+        _suppressReadExceptionOnClose = suppress;
+    }
 }
